@@ -1,7 +1,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2011 The OpenLDAP Foundation.
+ * Copyright 1998-2014 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -426,9 +426,6 @@ ldap_sasl_interactive_bind(
 	char *smechs = NULL;
 	int rc;
 
-#if defined( HAVE_CYRUS_SASL )
-	LDAP_MUTEX_LOCK( &ldap_int_sasl_mutex );
-#endif
 #ifdef LDAP_CONNECTIONLESS
 	if( LDAP_IS_UDP(ld) ) {
 		/* Just force it to simple bind, silly to make the user
@@ -476,9 +473,6 @@ ldap_sasl_interactive_bind(
 		flags, interact, defaults, result, rmech, msgid );
 
 done:
-#if defined( HAVE_CYRUS_SASL )
-	LDAP_MUTEX_UNLOCK( &ldap_int_sasl_mutex );
-#endif
 	if ( smechs ) LDAP_FREE( smechs );
 
 	return rc;
