@@ -1,8 +1,8 @@
 /* ava.c - routines for dealing with attribute value assertions */
-/* $OpenLDAP: pkg/ldap/servers/slapd/ava.c,v 1.45.2.3 2008/02/11 23:26:43 kurt Exp $ */
+/* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2008 The OpenLDAP Foundation.
+ * Copyright 1998-2014 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -98,7 +98,7 @@ get_ava(
 			aa->aa_desc = slap_bv2tmp_ad( &type, op->o_tmpmemctx );
 			ber_dupbv_x( &aa->aa_value, &value, op->o_tmpmemctx );
 			f->f_ava = aa;
-			return rc;
+			return LDAP_SUCCESS;
 		}
 	}
 
@@ -111,6 +111,7 @@ get_ava(
 		Debug( LDAP_DEBUG_FILTER,
 		"get_ava: illegal value for attributeType %s\n", type.bv_val, 0, 0 );
 		ber_dupbv_x( &aa->aa_value, &value, op->o_tmpmemctx );
+		*text = NULL;
 		rc = LDAP_SUCCESS;
 	}
 
@@ -121,7 +122,7 @@ get_ava(
 			rc = get_aliased_filter_aa ( op, aa, a_alias, text );
 			if( rc != LDAP_SUCCESS ) {
 				Debug( LDAP_DEBUG_FILTER,
-						"get_ava:Invalid Attribute Aliasing\n", 0, 0, 0 );
+						"get_ava: Invalid Attribute Aliasing\n", 0, 0, 0 );
 				return rc;
 			}
 		}
