@@ -1,8 +1,8 @@
 /* conn.c - deal with connection subsystem */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-monitor/conn.c,v 1.72.2.12 2011/01/04 23:50:37 kurt Exp $ */
+/* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2001-2011 The OpenLDAP Foundation.
+ * Copyright 2001-2014 The OpenLDAP Foundation.
  * Portions Copyright 2001-2003 Pierangelo Masarati.
  * All rights reserved.
  *
@@ -77,7 +77,7 @@ monitor_subsys_conn_init(
 	 */
 	BER_BVSTR( &bv, "cn=Max File Descriptors" );
 	e = monitor_entry_stub( &ms->mss_dn, &ms->mss_ndn, &bv,
-		mi->mi_oc_monitorCounterObject, mi, NULL, NULL );
+		mi->mi_oc_monitorCounterObject, NULL, NULL );
 	
 	if ( e == NULL ) {
 		Debug( LDAP_DEBUG_ANY,
@@ -122,7 +122,7 @@ monitor_subsys_conn_init(
 	 */
 	BER_BVSTR( &bv, "cn=Total" );
 	e = monitor_entry_stub( &ms->mss_dn, &ms->mss_ndn, &bv,
-		mi->mi_oc_monitorCounterObject, mi, NULL, NULL );
+		mi->mi_oc_monitorCounterObject, NULL, NULL );
 	
 	if ( e == NULL ) {
 		Debug( LDAP_DEBUG_ANY,
@@ -161,7 +161,7 @@ monitor_subsys_conn_init(
 	 */
 	BER_BVSTR( &bv, "cn=Current" );
 	e = monitor_entry_stub( &ms->mss_dn, &ms->mss_ndn, &bv,
-		mi->mi_oc_monitorCounterObject, mi, NULL, NULL );
+		mi->mi_oc_monitorCounterObject, NULL, NULL );
 
 	if ( e == NULL ) {
 		Debug( LDAP_DEBUG_ANY,
@@ -223,7 +223,7 @@ monitor_subsys_conn_update(
 
 	} else if ( dn_match( &rdn, &current_bv ) ) {
 		Connection	*c;
-		int		connindex;
+		ber_socket_t	connindex;
 
 		for ( n = 0, c = connection_first( &connindex );
 				c != NULL;
@@ -292,7 +292,7 @@ conn_create(
 		"cn=Connection %ld", c->c_connid );
 	bv.bv_val = buf;
 	e = monitor_entry_stub( &ms->mss_dn, &ms->mss_ndn, &bv, 
-		mi->mi_oc_monitorConnection, mi, &ctmbv, &mtmbv );
+		mi->mi_oc_monitorConnection, &ctmbv, &mtmbv );
 
 	if ( e == NULL) {
 		Debug( LDAP_DEBUG_ANY,
@@ -445,7 +445,7 @@ monitor_subsys_conn_create(
 
 	if ( ndn == NULL ) {
 		Connection	*c;
-		int		connindex;
+		ber_socket_t	connindex;
 		Entry		*e = NULL,
 				*e_tmp = NULL;
 
@@ -481,7 +481,7 @@ monitor_subsys_conn_create(
 
 	} else {
 		Connection		*c;
-		int			connindex;
+		ber_socket_t		connindex;
 		unsigned long 		connid;
 		char			*next = NULL;
 		static struct berval	nconn_bv = BER_BVC( "cn=connection " );
