@@ -1,8 +1,8 @@
 /* null.c - the null backend */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-null/null.c,v 1.18.2.8 2009/01/22 00:01:09 kurt Exp $ */
+/* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2002-2009 The OpenLDAP Foundation.
+ * Copyright 2002-2014 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,6 +31,28 @@ struct null_info {
 	ID	ni_nextid;
 };
 
+static ConfigDriver null_cf_gen;
+
+static ConfigTable nullcfg[] = {
+	{ "bind", "true|FALSE", 1, 2, 0, ARG_ON_OFF|ARG_MAGIC,
+		null_cf_gen,
+		"( OLcfgDbAt:8.1 NAME 'olcDbBindAllowed' "
+		"DESC 'Allow binds to this database' "
+		"SYNTAX OMsBoolean SINGLE-VALUE )", NULL, NULL },
+	{ NULL, NULL, 0, 0, 0, ARG_IGNORED,
+		NULL, NULL, NULL, NULL }
+};
+
+static ConfigOCs nullocs[] = {
+	{ "( OLcfgDbOc:8.1 "
+		"NAME 'olcNullConfig' "
+		"DESC 'Null backend ocnfiguration' "
+		"SUP olcDatabaseConfig "
+		"MAY ( olcDbBindAllowed ) )",
+		Cft_Database, nullcfg },
+	{ NULL, 0, NULL }
+};
+
 
 /* LDAP operations */
 
@@ -234,8 +256,6 @@ null_back_entry_get(
 	int rw,
 	Entry **ent )
 {
-	assert( *ent == NULL );
-
 	/* don't admit the object isn't there */
 	return oc || at ? LDAP_NO_SUCH_ATTRIBUTE : LDAP_BUSY;
 }
@@ -256,6 +276,12 @@ null_tool_entry_close( BackendDB *be )
 	return 0;
 }
 
+static ID
+null_tool_entry_first_x( BackendDB *be, struct berval *base, int scope, Filter *f )
+{
+	return NOID;
+}
+
 static ID
 null_tool_entry_next( BackendDB *be )
 {
@@ -285,37 +311,20 @@ null_tool_entry_put( BackendDB *be, Entry *e, struct berval *text )
 /* Setup */
 
 static int
-null_back_db_config(
-	BackendDB	*be,
-	const char	*fname,
-	int		lineno,
-	int		argc,
-	char		**argv )
+null_cf_gen( ConfigArgs *c )
 {
-	struct null_info *ni = (struct null_info *) be->be_private;
+	struct null_info *ni = (struct null_info *) c->be->be_private;
 
-	if ( ni == NULL ) {
-		fprintf( stderr, "%s: line %d: null database info is null!\n",
-			fname, lineno );
-		return 1;
+	if ( c->op == SLAP_CONFIG_EMIT ) {
+		c->value_int = ni->ni_bind_allowed;
+		return LDAP_SUCCESS;
+	} else if ( c->op == LDAP_MOD_DELETE ) {
+		ni->ni_bind_allowed = 0;
+		return LDAP_SUCCESS;
 	}
 
-	/* bind requests allowed */
-	if ( strcasecmp( argv[0], "bind" ) == 0 ) {
-		if ( argc < 2 ) {
-			fprintf( stderr,
-	"%s: line %d: missing <on/off> in \"bind <on/off>\" line\n",
-			         fname, lineno );
-			return 1;
-		}
-		ni->ni_bind_allowed = strcasecmp( argv[1], "off" );
-
-	/* anything else */
-	} else {
-		return SLAP_CONF_UNKNOWN;
-	}
-
-	return 0;
+	ni->ni_bind_allowed = c->value_int;
+	return LDAP_SUCCESS;
 }
 
 static int
@@ -325,6 +334,7 @@ null_back_db_init( BackendDB *be, ConfigReply *cr )
 	ni->ni_bind_allowed = 0;
 	ni->ni_nextid = 1;
 	be->be_private = ni;
+	be->be_cf_ocs = be->bd_info->bi_cf_ocs;
 	return 0;
 }
 
@@ -368,7 +378,7 @@ null_back_initialize( BackendInfo *bi )
 	bi->bi_destroy = 0;
 
 	bi->bi_db_init = null_back_db_init;
-	bi->bi_db_config = null_back_db_config;
+	bi->bi_db_config = config_generic_wrapper;
 	bi->bi_db_open = 0;
 	bi->bi_db_close = 0;
 	bi->bi_db_destroy = null_back_db_destroy;
@@ -394,12 +404,14 @@ null_back_initialize( BackendInfo *bi )
 
 	bi->bi_tool_entry_open = null_tool_entry_open;
 	bi->bi_tool_entry_close = null_tool_entry_close;
-	bi->bi_tool_entry_first = null_tool_entry_next;
+	bi->bi_tool_entry_first = backend_tool_entry_first;
+	bi->bi_tool_entry_first_x = null_tool_entry_first_x;
 	bi->bi_tool_entry_next = null_tool_entry_next;
 	bi->bi_tool_entry_get = null_tool_entry_get;
 	bi->bi_tool_entry_put = null_tool_entry_put;
 
-	return 0;
+	bi->bi_cf_ocs = nullocs;
+	return config_register_schema( nullcfg, nullocs );
 }
 
 #if SLAPD_NULL == SLAPD_MOD_DYNAMIC
